#!/bin/bash
#
#   Note this script contains the logic to install various third party tools.
#   It is shared between several of the infra pipeline images, and should be called once for each tool that
#   is to be installed. This is to ensure that each tool is installed in a single layer of the container,
#   which is not essential, but helps readability and local build caching.
#
#   (You can of course still call/chain this script multiple times if you must have a single layer ;-)
set -e
set -o xtrace

# check_var asserts that a given variable is set and not blank, or it exits with helpful text
check_var() {
    # Get the variable's value using indirect reference
    local value="${!1}"
    if [ -z "$value" ]; then
        echo "$INSTALL_TARGET install requires variable '$1' to be set. Please check the ARGS in the Dockerfile"
        exit 1
    fi
}


# Validate that we were given a target to install
if [  -z "$1" ]; then
    echo "This script must be called with the tool to install as the second parameter. eg './installers.sh helm' "
    exit 1
fi

INSTALL_TARGET=$1

# Now we can select and run the required installer
echo -e "\nInstalling: $INSTALL_TARGET\n"
cd /tmp
case $INSTALL_TARGET in
    aws-cli)
        check_var AWSCLI_VERSION
        curl "https://awscli.amazonaws.com/awscli-exe-linux-x86_64-${AWSCLI_VERSION}.zip" -o /tmp/awscliv2.zip
        unzip /tmp/awscliv2.zip
        rm -f /tmp/awscliv2.zip
        ./aws/install
    ;;
    azure-cli)
        # Use this sparingly its going to add a 1gb of crud to any image which installs it
        check_var AZURE_CLI_VERSION
        check_var AZURE_FRONTDOOR_VERSION
        curl -sL https://packages.microsoft.com/keys/microsoft.asc |gpg --dearmor | tee /etc/apt/trusted.gpg.d/microsoft.asc.gpg
        echo "deb [arch=amd64] https://packages.microsoft.com/repos/azure-cli/ $(lsb_release -cs) main" | tee /etc/apt/sources.list.d/azure-cli.list
        apt-get update && apt-get install --no-install-recommends --yes azure-cli=${AZURE_CLI_VERSION}
        #          Unfortunately the SDK Trim is a bit heavy handed, we need to use the whole thing :-( - Leaving this in comments in case
        #          this method can be revived and refined later
        #        pip3 install azure-cli==${AZURE_CLI_VERSION}
        #        pip3 install git+https://github.com/clumio-code/azure-sdk-trim@v0.1.0#egg=azure-sdk-trim
        #        azure-sdk-trim
        az extension add --name front-door --version ${AZURE_FRONTDOOR_VERSION} --upgrade --yes
        apt-get clean
        rm -rf /var/lib/apt/lists/*
    ;;
    conftest)
        # Check parameters
        check_var CONFTEST_VERSION
        
        # Download and extract Conftext
        curl -LO https://github.com/open-policy-agent/conftest/releases/download/v${CONFTEST_VERSION}/conftest_${CONFTEST_VERSION}_Linux_x86_64.tar.gz
        tar -xvf conftest_${CONFTEST_VERSION}_Linux_x86_64.tar.gz
        
        # Move binary and tidy up
        mv conftest /usr/local/bin
        rm conftest_${CONFTEST_VERSION}_Linux_x86_64.tar.gz
    ;;
    
    goss)
        check_var GOSS_VERSION
        curl -L "https://github.com/aelsabbahy/goss/releases/download/${GOSS_VERSION}/goss-linux-amd64" -o /usr/local/bin/goss
        chmod 755 /usr/local/bin/goss
    ;;
    
    helm)
        # Check parameters
        check_var HELM_LOCATION
        check_var HELM_FILENAME
        
        # Install Helm & helm-secrets plugin
        cd /tmp
        curl -LO ${HELM_LOCATION}/${HELM_FILENAME}
        tar zxvf ${HELM_FILENAME}
        
        # Move binary, remove install files
        mv linux-amd64/helm /usr/local/bin/
        rm ${HELM_FILENAME}
        rm -rf linux-amd64
        
        # Install helm secrets
        helm plugin install https://github.com/futuresimple/helm-secrets
    ;;
    kops)
        # Check parameters
        check_var KOPS_VERSION
        curl -L https://github.com/kubernetes/kops/releases/download/${KOPS_VERSION}/kops-linux-amd64 -o /usr/local/bin/kops
        chmod 755 /usr/local/bin/kops
    ;;
    
    kubectl)
        check_var KUBECTL_VERSION
        curl -L "https://storage.googleapis.com/kubernetes-release/release/${KUBECTL_VERSION}/bin/linux/amd64/kubectl" -o /usr/local/bin/kubectl
        chmod 755 /usr/local/bin/kubectl
    ;;
    
    kubeval)
        # Check parameters
        check_var KUBEVAL_VERSION
        
        # Download and extract kubeval
        curl -L https://github.com/instrumenta/kubeval/releases/download/${KUBEVAL_VERSION}/kubeval-linux-amd64.tar.gz -o /tmp/kubeval.tar.gz
        tar zxvf /tmp/kubeval.tar.gz
        
        # Move binary and tidy up
        mv kubeval /usr/local/bin
        rm /tmp/kubeval.tar.gz
        chmod 755 /usr/local/bin/kubeval
    ;;
    
    platform-automation-libs)
        check_var GITLAB_HOST
        check_var CI_JOB_TOKEN
        check_var AUTOMATION_TOOLS_LIBS_VERSION
        check_var AUTOMATION_TOOLS_LIBS_GIT_URL
        
        echo -e "machine ${GITLAB_HOST}\nlogin gitlab-ci-token\npassword ${CI_JOB_TOKEN}" > ~/.netrc
        echo "Requested version of automation libraries is '$AUTOMATION_TOOLS_LIBS_VERSION'"
        
        git clone $(echo "${AUTOMATION_TOOLS_LIBS_GIT_URL}" | sed 's:^git+::') platform-automation-libs
        # Clone platform automation tools repo and get a list of branches and tags
        git -C ./platform-automation-libs show-ref | awk '{ print $2 }' | grep '^refs/remotes/origin/\|^refs/heads/\|^refs/tags/'  > /tmp/branches_and_tags
        # Filter that list for those that match the specified automation libs version
        cat /tmp/branches_and_tags | sed 's:^refs/remotes/origin/:refs/:' | sed 's:^refs/heads/:refs/:' | grep "^refs/$AUTOMATION_TOOLS_LIBS_VERSION" | sed 's:^refs/tags/:refs/:' | sed 's:^refs/::' > /tmp/matching_versions
        # Sort those matches in ascending order and return the last (greatest) one
        AUTOMATION_TOOLS_LIBS_VERSION_MATCH=$(cat /tmp/matching_versions | sort --version-sort | tail -n 1)
        
        if [ ! -n "$AUTOMATION_TOOLS_LIBS_VERSION_MATCH" ]; then
            echo "Unable to resolve automation tools version for input $AUTOMATION_TOOLS_LIBS_VERSION"
            exit 1
        fi
        
        echo "Resolved version of automation libraries is '$AUTOMATION_TOOLS_LIBS_VERSION_MATCH'"
        # Cleanup
        rm -f /tmp/branches_and_tags
        rm -f /tmp/matching_versions
        rm -rf platform-automation-libs
        
        # Install platform automation libraries via pip
        python3 -m pip install ${AUTOMATION_TOOLS_LIBS_GIT_URL}@${AUTOMATION_TOOLS_LIBS_VERSION_MATCH}
    ;;
    popeye)
        # Check parameters
        check_var POPEYE_VERSION
        
        curl -L https://github.com/derailed/popeye/releases/download/${POPEYE_VERSION}/popeye_Linux_x86_64.tar.gz -o /tmp/popeye.tar.gz
        cd /tmp; tar zxvf popeye.tar.gz
        mv /tmp/popeye /usr/local/bin/popeye
        chmod 755 /usr/local/bin/popeye
        
        # Cleanup tar
        rm popeye.tar.gz
    ;;
    terraform)
        check_var TERRAFORM_VERSION
        
        # Download terraform
        TF_FILE=terraform_${TERRAFORM_VERSION}_linux_amd64.zip
        curl https://releases.hashicorp.com/terraform/${TERRAFORM_VERSION}/${TF_FILE} \
        -o /tmp/${TF_FILE}
        
        # Unzip and move terraform binary
        unzip /tmp/${TF_FILE}
        mv terraform /usr/local/bin
        chmod 755 /usr/local/bin/terraform
        
        # Cleanup
        rm -f /tmp/${TF_FILE}
    ;;
    yq)
        check_var YQ_VERSION
        curl -L https://github.com/mikefarah/yq/releases/download/${YQ_VERSION}/yq_linux_amd64 -o /usr/local/bin/yq
        chmod +x /usr/local/bin/yq
    ;;
    firebase)
        gem install bundler
        gem install fastlane -v 2.217.0
        gem install aws-sdk-s3 -v 1.139.0
        gem install json -v 2.6.3
        gem install rexml -v 3.2.6
        gem install aws-partitions -v 1.856.0
        gem install webrick -v 1.8.1
        gem install rake -v 13.1.0
        gem install fastlane-plugin-firebase_app_distribution -v 0.7.4
        curl -sL https://firebase.tools | bash
        curl -L https://gitlab.com/gitlab-org/cli/-/releases/v1.35.0/downloads/glab_1.35.0_Linux_x86_64.deb -o /usr/local/glab.deb
        dpkg -i /usr/local/glab.deb
        rm -rf /usr/local/glab.deb
    ;;
    glab)
        curl -L https://gitlab.com/gitlab-org/cli/-/releases/v1.35.0/downloads/glab_1.35.0_Linux_x86_64.deb -o /usr/local/glab.deb
        dpkg -i /usr/local/glab.deb
        rm -rf /usr/local/glab.deb
    ;;
    *)
        echo "Invalid choice of tool to install. Please check name."
        exit 1
    ;;
esac
